@@ -1,15 +1,22 @@
-#' Get the code identifiers of the children of a code with a given identifier
+#' Get the code identifiers a code's descendents
+#'
+#' Get the code identifiers of all children, or all descendents (i.e. including
+#' grand-children, grand-grand-children, etc) of a code with a given identifier.
 #'
 #' @param x The parsed sources object
 #' @param parentCodeId The code identifier of the parent code
-#' @param returnNodes Set to `TRUE` to return a list of nodes, not just the
-#' code identifiers
+#' @param returnNodes For `get_childCodeIds()`, set this to `TRUE` to return
+#' a list of nodes, not just the code identifiers.
+#' @param includeParentCode Whether to include the parent code
+#' identifier in the result
 #'
 #' @return A character vector with code identifiers (or a list of nodes)
+#' @rdname get_childCodeIds
 #' @export
 get_childCodeIds <- function(x,
                              parentCodeId,
-                             returnNodes = FALSE) {
+                             returnNodes = FALSE,
+                             includeParentCode = FALSE) {
 
   if ((!inherits(x, "rock_parsedSources")) &&
       (!inherits(x, "rock_parsedSource"))) { ### Added this, might be wrong
@@ -27,14 +34,19 @@ get_childCodeIds <- function(x,
   if (inherits(x, "rock_parsedSource")) {
     node <- NULL;
     for (i in seq_along(x$inductiveCodeTrees)) {
-      if (!is.null(x$inductiveCodeTrees[[i]])) {
+      node <- NULL;
+      if ((!is.null(x$inductiveCodeTrees[[i]])) && is.null(node)) {
         node <-
           data.tree::FindNode(x$inductiveCodeTrees[[i]], parentCodeId);
       }
     }
-  } else {
+  } else if (inherits(x, "rock_parsedSources")) {
     node <-
       data.tree::FindNode(x$fullyMergedCodeTrees, parentCodeId);
+  } else {
+    stop("As `x`, you passed '",
+         substitute(deparse(x)), "', but this does not have class ",
+         "`rock_parsedSource` or `rock_parsedSources`.");
   }
 
   if (is.null(node)) {
@@ -45,6 +57,10 @@ get_childCodeIds <- function(x,
 
   childNodes <- node$children;
 
+  if (includeParentCode) {
+    childNodes <- c(parentCodeId, childNodes);
+  }
+
   if (returnNodes) {
     return(childNodes);
   } else {
