@@ -49,6 +49,11 @@
 #' @rdname parsing_sources
 #' @aliases parsing_sources parse_source parse_sources print.rock_parsedSource
 #'
+#' @return For `rock::parse_source()`, an object of class `rock_parsedSource`;
+#' for `rock::parse_sources()`, an object of class `rock_parsedSources`. These
+#' objects contain the original source(s) as well as the final data frame with
+#' utterances and codes, as well as the code structures.
+#'
 #' @examples ### Get path to example source
 #' examplePath <-
 #'   system.file("extdata", package="rock");
@@ -69,8 +74,12 @@
 #' parsedExample <- rock::parse_source(exampleFile,
 #'                                     silent=FALSE);
 #'
-#' ### Parse all example sources in that directory
-#' parsedExamples <- rock::parse_sources(examplePath);
+#' ### Parse as selection of example sources in that directory
+#' parsedExamples <-
+#'   rock::parse_sources(
+#'     examplePath,
+#'     regex = "(test|example)(.txt|.rock)"
+#'   );
 #'
 #' ### Show combined inductive code tree for the codes
 #' ### extracted with the regular expression specified with
@@ -91,15 +100,21 @@ parse_source <- function(text,
 
   codeRegexes <- rock::opts$get('codeRegexes');
   idRegexes <- rock::opts$get('idRegexes');
+  ciid_columnsToCopy <- rock::opts$get('ciid_columnsToCopy');
+  anchorRegex <- rock::opts$get('anchorRegex');
+  anchorsCol <- rock::opts$get('anchorsCol');
+  classInstanceRegex <- rock::opts$get('classInstanceRegex');
   codeValueRegexes <- rock::opts$get('codeValueRegexes');
   sectionRegexes <- rock::opts$get('sectionRegexes');
   uidRegex <- rock::opts$get('uidRegex');
   autoGenerateIds <- rock::opts$get('autoGenerateIds');
-  persistentIds <- rock::opts$get('persistentIds');
+  ### Obsolete now all class instance identifiers are persistent
+  # persistentIds <- rock::opts$get(persistentIds);
   noCodes <- rock::opts$get('noCodes');
   inductiveCodingHierarchyMarker <- rock::opts$get('inductiveCodingHierarchyMarker');
   attributeContainers <- rock::opts$get('attributeContainers');
   networkContainers <- rock::opts$get('networkContainers');
+  aestheticContainers <- rock::opts$get('aestheticContainers');
   codesContainers <- rock::opts$get('codesContainers');
   sectionBreakContainers <- rock::opts$get('sectionBreakContainers');
   delimiterRegEx <- rock::opts$get('delimiterRegEx');
@@ -215,6 +230,17 @@ parse_source <- function(text,
     res$rawDeductiveCodes <-
       yum::load_and_simplify(yamlFragments=res$yamlFragments,
                              select=paste0(codesContainers, collapse="|"));
+
+    # ### Store data frame with all info about deductive codes
+    # res$deductiveCodeDfs <-
+    #   lapply(
+    #     parsedSource$rawDeductiveCodes,
+    #     get_dataframe_from_nested_list
+    #   );
+    #
+    # names(res$deductiveCodeDfs) <-
+    #   names(res$rawDeductiveCodes);
+
     ### Get all deductive code ids
     res$deductiveCodes <-
       get_deductive_code_values(res$rawDeductiveCodes,
@@ -251,12 +277,12 @@ parse_source <- function(text,
       "Looking for network configuration.\n",
       silent = silent
     );
-    res$networkConfig <-
+    res$aestheticConfig <-
       yum::load_and_simplify(yamlFragments=res$yamlFragments,
-                             select=paste0(networkContainers, collapse="|"));
+                             select=paste0(aestheticContainers, collapse="|"));
     msg(
-      "Read ", length(unlist(res$networkConfig)),
-      " network confguration specifications.\n",
+      "Read ", length(unlist(res$aestheticConfig)),
+      " aesthetic specifications.\n",
       silent = silent
     );
 
@@ -276,7 +302,7 @@ parse_source <- function(text,
     res$deductiveCodes <- NA;
     res$deductiveCodeTrees <- NA;
     res$sectionBreakRegexes <- NA;
-    res$networkConfig <- NA;
+    res$aestheticConfig <- NA;
   }
 
   ###---------------------------------------------------------------------------
@@ -396,88 +422,296 @@ parse_source <- function(text,
     }
   }
 
+  ###---------------------------------------------------------------------------
+  ### Process specified class instance identifiers
   ###---------------------------------------------------------------------------
 
-  ### Process identifiers
-  if (!is.null(idRegexes) && length(idRegexes) > 0) {
-    for (idRegex in names(idRegexes)) {
+  # if (!is.null(idRegexes) && length(idRegexes) > 0) {
+  #   for (idRegex in names(idRegexes)) {
+  #
+  #     ### Get a list of matches
+  #     ids <-
+  #       regmatches(x,
+  #                  gregexpr(idRegexes[idRegex], x));
+  #
+  #     ### Check whether there are multiple matches
+  #     multipleIds <-
+  #       which(unlist(lapply(ids, length))>1);
+  #     if (length(multipleIds) > 0) {
+  #       warning(glue::glue("Multiple class instance identifiers matching '{idRegex}' found in the following utterances:\n",
+  #                      paste0(x[multipleIds],
+  #                             collapse="\n"),
+  #                      "\n\nOnly using the first  class instanceidentifier for each utterance, removing and ignoring the rest!"));
+  #       ids <-
+  #         lapply(ids, utils::head, 1);
+  #     }
+  #
+  #     ### Clean identifiers (i.e. only retain identifier content itself)
+  #     ids <-
+  #       lapply(ids, gsub, pattern=idRegexes[idRegex], replacement="\\1");
+  #
+  #     ### Set "no_id" for utterances without id
+  #     ids <-
+  #       ifelse(unlist(lapply(ids,
+  #                            length)),
+  #              ids,
+  #              "no_id");
+  #
+  #     ### Convert from a list to a vector
+  #     ids <- unlist(ids);
+  #
+  #     if (length(ids) > 1) {
+  #
+  #       ### Implement 'identifier persistence' by copying the
+  #       ### identifier of the previous utterance if the identifier
+  #       ### is not set - can't be done using vectorization as identifiers
+  #       ### have to carry over sequentially.
+  #       # if (idRegex %in% persistentIds) {
+  #       #   rawIds <- ids;
+  #       #   for (i in 2:length(ids)) {
+  #       #     if ((ids[i] == "no_id")) {
+  #       #       ids[i] <- ids[i-1];
+  #       #     }
+  #       #   }
+  #       # }
+  #
+  #       ### 2022-10-07 --- Decision Szilvia & GJ: change ROCK standard such
+  #       ###                that all class instance identifiers are always
+  #       ###                persistent.
+  #       rawIds <- ids;
+  #       for (i in 2:length(ids)) {
+  #         if ((ids[i] == "no_id")) {
+  #           ids[i] <- ids[i-1];
+  #         }
+  #       }
+  #
+  #     } else {
+  #       ids = "no_id";
+  #     }
+  #
+  #     ### Check whether any matches were found
+  #     if (!(all(ids=="no_id"))) {
+  #       ### Generate identifiers for ids without identifier
+  #       if (idRegex %in% autoGenerateIds) {
+  #         ids[ids=="no_id"] <-
+  #           paste0("autogenerated_id_",
+  #                  1:(sum(ids=="no_id")));
+  #       }
+  #       ### Store identifiers in sourceDf
+  #       sourceDf[, idRegex] <-
+  #         ids;
+  #       # if (idRegex %in% persistentIds) {
+  #       #   sourceDf[, paste0(idRegex, "_raw")] <-
+  #       #     rawIds;
+  #       # }
+  #       ### 2022-10-07 --- Decision Szilvia & GJ: change ROCK standard such
+  #       ###                that all class instance identifiers are always
+  #       ###                persistent.
+  #       sourceDf[, paste0(idRegex, "_raw")] <-
+  #         rawIds;
+  #
+  #     }
+  #   }
+  # }
 
-      ### Get a list of matches
-      ids <-
-        regmatches(x,
-                   gregexpr(idRegexes[idRegex], x));
+  ###---------------------------------------------------------------------------
+  ### Process unspecified (generic) class instance identifiers
+  ###---------------------------------------------------------------------------
 
-      ### Check whether there are multiple matches
-      multipleIds <-
-        which(unlist(lapply(ids, length))>1);
-      if (length(multipleIds) > 0) {
-        warning(glue::glue("Multiple identifiers matching '{idRegex}' found in the following utterances:\n",
-                       paste0(x[multipleIds],
-                              collapse="\n"),
-                       "\n\nOnly using the first identifier for each utterance, removing and ignoring the rest!"));
-        ids <-
-          lapply(ids, utils::head, 1);
-      }
+  classIdMatches <-
+    regmatches(x,
+               gregexpr(classInstanceRegex,
+                        x,
+                        perl = TRUE));
 
-      ### Clean identifiers (i.e. only retain identifier content itself)
-      ids <-
-        lapply(ids, gsub, pattern=idRegexes[idRegex], replacement="\\1");
+  if (all(unlist(lapply(classIdMatches, length)) == 0)) {
 
-      ### Set "no_id" for utterances without id
-      ids <-
-        ifelse(unlist(lapply(ids,
-                             length)),
-               ids,
-               "no_id");
+    msg(
+      "No UIDs or class instance identifiers found.\n",
+      silent = silent
+    );
 
-      ### Convert from a list to a vector
-      ids <- unlist(ids);
+  } else {
 
-      if (length(ids) > 1) {
-        ### Implement 'identifier persistence' by copying the
-        ### identifier of the previous utterance if the identifier
-        ### is not set - can't be done using vectorization as identifiers
-        ### have to carry over sequentially.
-        if (idRegex %in% persistentIds) {
-          rawIds <- ids;
-          for (i in 2:length(ids)) {
-            if ((ids[i] == "no_id")) {
-              ids[i] <- ids[i-1];
+    msg(
+      "Found UIDS or class instance identifiers: commencing to process.\n",
+      silent = silent
+    );
+
+    classIdMatches <-
+      lapply(
+        classIdMatches,
+        function(x) {
+          return(list(
+            gsub(classInstanceRegex, "\\1", x, perl = TRUE),
+            gsub(classInstanceRegex, "\\2", x, perl = TRUE)
+          ));
+        }
+      );
+
+    namedClassIdMatches <-
+      lapply(
+        classIdMatches,
+        function(x) {
+          return(
+            stats::setNames(
+              x[[2]],
+              x[[1]]
+            )
+          );
+        }
+      );
+
+    unspecifiedClasses <-
+      unique(
+        unlist(
+          lapply(
+            classIdMatches,
+            function(x) {
+              if (length(x) == 0) {
+                return(NULL)
+              } else {
+                return(x[[1]]);
+              }
             }
+          )
+        )
+      );
+
+    unspecifiedClasses <- setdiff(unspecifiedClasses, c("uid"));
+
+    # if (grepl("example-2.rock", text)) {
+    #   browser();
+    # }
+
+    if (length(unspecifiedClasses) == 0) {
+
+      msg(
+        "No unspecified class instance identifiers found.\n",
+        silent = silent
+      );
+
+    } else {
+
+      msg(
+        "Found class instance identifiers: commencing to process.\n",
+        silent = silent
+      );
+
+      unspecifiedClassInstanceIdentifierDf_raw <-
+        rbind_df_list(
+          lapply(
+            namedClassIdMatches,
+            function(x) {
+              if ((length(x) == 0) || (all(tolower(names(x)) == "uid"))) {
+                return(
+                  stats::setNames(
+                    data.frame(t(rep(NA, length(unspecifiedClasses)))),
+                    unspecifiedClasses
+                  )
+                );
+              } else {
+                cols <- names(x)[names(x) %in% unspecifiedClasses];
+                return(as.data.frame(as.list(x))[, cols, drop=FALSE]);
+              }
+            }
+          )
+        );
+
+      unspecifiedClassInstanceIdentifierDf <-
+        lapply(
+          unspecifiedClassInstanceIdentifierDf_raw,
+          function(x) {
+            x <-
+              ifelse(is.na(x) | (nchar(x) == 0),
+                     "no_id",
+                     x);
+            for (i in 2:length(x)) {
+              if ((x[i] == "no_id")) {
+                x[i] <- x[i-1];
+              }
+            }
+            return(x);
           }
-        }
-      } else {
-        ids = "no_id";
+        );
+
+      if ((!(all(unspecifiedClasses %in%
+             names(unspecifiedClassInstanceIdentifierDf))) ||
+          (!(all(names(unspecifiedClassInstanceIdentifierDf) %in%
+             unspecifiedClasses))))) {
+        stop("Inconsistency in column names");
       }
 
-      ### Check whether any matches were found
-      if (!(all(ids=="no_id"))) {
-        ### Generate identifiers for ids without identifier
-        if (idRegex %in% autoGenerateIds) {
-          ids[ids=="no_id"] <-
-            paste0("autogenerated_id_",
-                   1:(sum(ids=="no_id")));
-        }
-        ### Store identifiers in sourceDf
-        sourceDf[, idRegex] <-
-          ids;
-        if (idRegex %in% persistentIds) {
-          sourceDf[, paste0(idRegex, "_raw")] <-
-            rawIds;
-        }
+      sourceDf[, unspecifiedClasses] <-
+        unspecifiedClassInstanceIdentifierDf
+      sourceDf[, paste0(unspecifiedClasses, "_raw")] <-
+        unspecifiedClassInstanceIdentifierDf_raw;
+
+      colsToCopy <- names(ciid_columnsToCopy) %in% unspecifiedClasses;
+
+      if (length(colsToCopy) > 0) {
+        colsToCopy <- ciid_columnsToCopy[colsToCopy];
+        sourceDf[, colsToCopy] <-
+          sourceDf[, names(colsToCopy)];
       }
+
+      msg(
+        "Processed ", length(unspecifiedClasses),
+        " class instance identifiers (", vecTxtQ(unspecifiedClasses), ").\n",
+        silent = silent
+      );
+
     }
+
   }
 
   ###---------------------------------------------------------------------------
-
   ### Delete identifiers and store clean version in sourceDf
+  ###---------------------------------------------------------------------------
+
   x <-
     gsub(paste0(idRegexes, collapse="|"),
          "",
-         x);
+         x,
+         perl = TRUE);
+  x <-
+    gsub(classInstanceRegex,
+         "",
+         x,
+         perl = TRUE);
+
   sourceDf$utterances_without_identifiers <- x;
 
+  ###---------------------------------------------------------------------------
+  ### Anchors
+  ###---------------------------------------------------------------------------
+
+  ### Get a list of matches
+  anchors <-
+    unlist(
+      lapply(
+        regmatches(x,
+                   gregexpr(anchorRegex, x)),
+        function(x) {
+          if (length(x) == 0) {
+            return("");
+          } else {
+            return(x);
+          }
+        }
+      )
+    );
+
+  sourceDf[, paste0(anchorsCol, "_raw")] <-
+    anchors;
+
+  sourceDf[, anchorsCol] <-
+    trimws(
+      gsub(anchorRegex,
+           "\\1",
+           anchors)
+    );
+
   ###---------------------------------------------------------------------------
   ### Process codes
   ###---------------------------------------------------------------------------
@@ -519,7 +753,7 @@ parse_source <- function(text,
           (length(codings[[codeRegex]]) > 0)) {
 
         ### Create an object with the intermediate objects
-        codeProcessing[[codeRegex]] <- list();
+        codeProcessing[[codeRegex]] <- list(matches = matches);
 
         ### Split the codes using the specified marker
         codeProcessing[[codeRegex]]$splitCodings <-
@@ -619,6 +853,12 @@ parse_source <- function(text,
         tryCatch({
           codeProcessing[[codeRegex]]$inductiveDiagrammeR <-
             data.tree::ToDiagrammeRGraph(codeProcessing[[codeRegex]]$inductiveCodeTrees);
+          codeProcessing[[codeRegex]]$inductiveDiagrammeR <-
+            do.call(
+              rock::apply_graph_theme,
+              c(list(graph = codeProcessing[[codeRegex]]$inductiveDiagrammeR),
+                rock::opts$get("theme_codeTreeDiagram"))
+            );
         }, error = function(e) {
           warning("Error issued by 'data.tree::ToDiagrammeRGraph' when converting '",
                   codeRegex, "' code tree: ", e$message, "\n\nClass and content:\n\n",
@@ -629,40 +869,42 @@ parse_source <- function(text,
                          collapse="\n"));
         });
 
-        ### Set matches for lines that did
-        ### not have a match to NA
-        cleanedMatches[unlist(lapply(matches, length))==0] <- NA;
+      } ### Changed this on 2022-11-27
 
-        ### Get presence of codes in utterances
-        occurrences[[codeRegex]] <-
-          lapply(get_leaf_codes(cleanedMatches,
-                                inductiveCodingHierarchyMarker=inductiveCodingHierarchyMarker),
-                 `%in%`,
-                 x=codeProcessing[[codeRegex]]$leafCodes);
+      ### Set matches for lines that did
+      ### not have a match to NA
+      cleanedMatches[unlist(lapply(matches, length))==0] <- NA;
 
-        ### Convert from logical to numeric
-        occurrenceCounts[[codeRegex]] <-
-          lapply(occurrences[[codeRegex]], as.numeric);
+      ### Get presence of codes in utterances
+      occurrences[[codeRegex]] <-
+        lapply(get_leaf_codes(cleanedMatches,
+                              inductiveCodingHierarchyMarker=inductiveCodingHierarchyMarker),
+               `%in%`,
+               x=codeProcessing[[codeRegex]]$leafCodes);
 
-        ### Add the codes as names
-        namedOccurrences[[codeRegex]] <-
-          lapply(occurrenceCounts[[codeRegex]],
-                 `names<-`,
-                 value <- codeProcessing[[codeRegex]]$leafCodes);
+      ### Convert from logical to numeric
+      occurrenceCounts[[codeRegex]] <-
+        lapply(occurrences[[codeRegex]], as.numeric);
 
-        ### Convert the lists to dataframes
-        sourceDf <-
-          cbind(sourceDf,
-                as.data.frame(do.call(rbind,
-                                      namedOccurrences[[codeRegex]])));
+      ### Add the codes as names
+      namedOccurrences[[codeRegex]] <-
+        lapply(occurrenceCounts[[codeRegex]],
+               `names<-`,
+               value <- codeProcessing[[codeRegex]]$leafCodes);
 
-        ### Delete codes from utterances
-        x <-
-          gsub(codeRegexes[codeRegex],
-               "",
-               x);
+      ### Convert the lists to dataframes
+      sourceDf <-
+        cbind(sourceDf,
+              as.data.frame(do.call(rbind,
+                                    namedOccurrences[[codeRegex]])));
 
-      }
+      ### Delete codes from utterances
+      x <-
+        gsub(codeRegexes[codeRegex],
+             "",
+             x);
+
+      ### }
     }
   }
 
@@ -908,7 +1150,7 @@ parse_source <- function(text,
               penwidth = res$networkCodes[[networkCodeRegex]]$coded_df$edge_weight
             );
 
-          if (!is.na(res$networkConfig)) {
+          if (!is.na(res$aestheticConfig)) {
 
             configName <- paste0("ROCK_", networkCodeRegex);
 
@@ -920,7 +1162,7 @@ parse_source <- function(text,
             configuredEdgeTypes <-
               unlist(
                 lapply(
-                  res$networkConfig[[configName]]$edges,
+                  res$aestheticConfig[[configName]]$edges,
                   function(x) {
                     if (is.null(x$type) || is.na(x$type) || (nchar(x$type) == 0)) {
                       return("no_type_specified");
@@ -933,20 +1175,28 @@ parse_source <- function(text,
 
             res$networkCodes[[networkCodeRegex]]$edgeConfig <-
               stats::setNames(
-                res$networkConfig[[configName]]$edges,
+                res$aestheticConfig[[configName]]$edges,
                 configuredEdgeTypes
               );
 
             for (currentType in uniqueTypes) {
 
-              res$networkCodes[[networkCodeRegex]]$coded_df[
-                res$networkCodes[[networkCodeRegex]]$coded_df$type ==
-                  currentType,
-                setdiff(names(res$networkCodes[[networkCodeRegex]]$edgeConfig[[currentType]]), "type")
-              ] <-
-                res$networkCodes[[networkCodeRegex]]$edgeConfig[[currentType]][
+              configuredColor <-
+                unlist(
+                  res$networkCodes[[networkCodeRegex]]$edgeConfig[[currentType]][
+                    setdiff(names(res$networkCodes[[networkCodeRegex]]$edgeConfig[[currentType]]), "type")
+                    ]
+                );
+
+              if (!is.null(configuredColor)) {
+
+                res$networkCodes[[networkCodeRegex]]$coded_df[
+                  which(res$networkCodes[[networkCodeRegex]]$coded_df$type ==
+                          currentType),
                   setdiff(names(res$networkCodes[[networkCodeRegex]]$edgeConfig[[currentType]]), "type")
-                ];
+                ] <-
+                  configuredColor;
+              }
 
             }
 
@@ -1185,14 +1435,29 @@ parse_source <- function(text,
   ### Clean the source dataframe by removing section break rows.
 
   if (nrow(sourceDf) > 0) {
+
     sourceDf$originalSequenceNr <- 1:nrow(sourceDf);
 
     cleanSourceDf <-
       sourceDf[!grepl(paste0(sectionRegexes, collapse="|"),
                       x), ];
 
+    ###-------------------------------------------------------------------------
+    ### Changed on 2022-09-13 because rows holding only a code were
+    ### also deleted (but shouldn't be)
+
     cleanSourceDf <-
-      cleanSourceDf[nchar(cleanSourceDf$utterances_clean)>0, ];
+      cleanSourceDf[!cleanSourceDf$sectionBreak_match, ];
+    cleanSourceDf <-
+      cleanSourceDf[nchar(cleanSourceDf$utterances_without_identifiers)>0, ];
+    cleanSourceDf <-
+      cleanSourceDf[nchar(cleanSourceDf$utterances_raw)>0, ];
+
+    ### Original, before 2022-09-13
+    # cleanSourceDf <-
+    #   cleanSourceDf[nchar(cleanSourceDf$utterances_clean)>0, ];
+    ###-------------------------------------------------------------------------
+
   } else {
     cleanSourceDf <- data.frame();
   }
@@ -1309,16 +1574,16 @@ parse_source <- function(text,
 
   }
 
-  ### Check for identifier column existence and convert to character
-  for (i in names(idRegexes)) {
-    if (i %in% names(res$mergedSourceDf)) {
-      res$mergedSourceDf[, i] <-
-        as.character(res$mergedSourceDf[, i]);
-    } else {
-      res$mergedSourceDf[, i] <-
-        rep("", nrow(res$mergedSourceDf));
-    }
-  }
+  # ### Check for identifier column existence and convert to character
+  # for (i in names(idRegexes)) {
+  #   if (i %in% names(res$mergedSourceDf)) {
+  #     res$mergedSourceDf[, i] <-
+  #       as.character(res$mergedSourceDf[, i]);
+  #   } else {
+  #     res$mergedSourceDf[, i] <-
+  #       rep("", nrow(res$mergedSourceDf));
+  #   }
+  # }
 
   ### Add codings and leaves only to the convenience list
   res$convenience$codings <- sort(unique(unlist(res$rawCodings)));
@@ -1355,7 +1620,10 @@ parse_source <- function(text,
              return(sort(unique(unname(unlist(x)))));
            });
 
-  if (!postponeDeductiveTreeBuilding && ("Node" %in% class(res$deductiveCodeTrees))) {
+  if (postponeDeductiveTreeBuilding) {
+    res$extendedDeductiveCodeTrees <- NA;
+    res$fullyMergedCodeTrees <- NA;
+  } else if (!postponeDeductiveTreeBuilding && ("Node" %in% class(res$deductiveCodeTrees))) {
     ### Merge inductive code tree into deductive code tree (currently only support
     ### for one deductive code tree)
     res$extendedDeductiveCodeTrees <-
@@ -1383,11 +1651,21 @@ parse_source <- function(text,
         }
       }
     }
-  } else {
+  } else if (!postponeDeductiveTreeBuilding) {
     res$extendedDeductiveCodeTrees <- NA;
-    res$fullyMergedCodeTrees <- NA;
+    res$fullyMergedCodeTrees <- res$inductiveCodeTrees;
   }
 
+
+  res$convenience$valid_inductiveCodeTrees <-
+    which(unlist(lapply(res$inductiveCodeTrees, is.environment)));
+
+  res$convenience$original_inductiveCodeTreeNames <-
+    names(res$inductiveCodeTrees);
+
+  res$inductiveCodeTrees <-
+    res$inductiveCodeTrees[res$convenience$valid_inductiveCodeTrees];
+
   if (!silent) {
     cat("\n\n");
   }
