@@ -15,7 +15,8 @@ parse_sources <- function(path,
   sectionRegexes <- rock::opts$get(sectionRegexes);
   uidRegex <- rock::opts$get(uidRegex);
   autoGenerateIds <- rock::opts$get(autoGenerateIds);
-  persistentIds <- rock::opts$get(persistentIds);
+  ### Obsolete now all class instance identifiers are persistent
+  # persistentIds <- rock::opts$get(persistentIds);
   noCodes <- rock::opts$get(noCodes);
   inductiveCodingHierarchyMarker <- rock::opts$get(inductiveCodingHierarchyMarker);
   attributeContainers <- rock::opts$get(attributeContainers);
@@ -228,21 +229,29 @@ parse_sources <- function(path,
           tree$root$Set(name = 'codes',
                         filterFun=function(x) x$isRoot);
           res <- data.tree::ToDiagrammeRGraph(tree);
+
           res <-
-            apply_graph_theme(res,
-                              c("layout", "dot", "graph"),
-                              c("rankdir", "LR", "graph"),
-                              c("outputorder", "edgesfirst", "graph"),
-                              c("fixedsize", "false", "node"),
-                              c("shape", "box", "node"),
-                              c("style", "rounded,filled", "node"),
-                              c("fontname", "Arial", "node"),
-                              c("color", "#000000", "node"),
-                              c("color", "#888888", "edge"),
-                              c("dir", "none", "edge"),
-                              c("headclip", "false", "edge"),
-                              c("tailclip", "false", "edge"),
-                              c("fillcolor", "#FFFFFF", "node"));
+            do.call(
+              rock::apply_graph_theme,
+              c(list(graph = res),
+                rock::opts$get("theme_codeTreeDiagram"))
+            );
+
+          # res <-
+          #   apply_graph_theme(res,
+          #                     c("layout", "dot", "graph"),
+          #                     c("rankdir", "LR", "graph"),
+          #                     c("outputorder", "edgesfirst", "graph"),
+          #                     c("fixedsize", "false", "node"),
+          #                     c("shape", "box", "node"),
+          #                     c("style", "rounded,filled", "node"),
+          #                     c("fontname", "Arial", "node"),
+          #                     c("color", "#000000", "node"),
+          #                     c("color", "#888888", "edge"),
+          #                     c("dir", "none", "edge"),
+          #                     c("headclip", "false", "edge"),
+          #                     c("tailclip", "false", "edge"),
+          #                     c("fillcolor", "#FFFFFF", "node"));
           return(res);
         }
       );
@@ -581,19 +590,25 @@ parse_sources <- function(path,
       data.tree::ToDiagrammeRGraph(res$deductiveCodeTrees);
 
     res$deductiveCodeTreeGraph <-
-      apply_graph_theme(res$deductiveCodeTreeGraph,
-                        c("layout", "dot", "graph"),
-                        c("rankdir", "LR", "graph"),
-                        c("outputorder", "edgesfirst", "graph"),
-                        c("fixedsize", "false", "node"),
-                        c("shape", "box", "node"),
-                        c("style", "rounded,filled", "node"),
-                        c("color", "#000000", "node"),
-                        c("color", "#888888", "edge"),
-                        c("dir", "none", "edge"),
-                        c("headclip", "false", "edge"),
-                        c("tailclip", "false", "edge"),
-                        c("fillcolor", "#FFFFFF", "node"));
+      do.call(
+        rock::apply_graph_theme,
+        c(list(graph = res$deductiveCodeTreeGraph),
+          rock::opts$get("theme_codeTreeDiagram"))
+      );
+
+      # apply_graph_theme(res$deductiveCodeTreeGraph,
+      #                   c("layout", "dot", "graph"),
+      #                   c("rankdir", "LR", "graph"),
+      #                   c("outputorder", "edgesfirst", "graph"),
+      #                   c("fixedsize", "false", "node"),
+      #                   c("shape", "box", "node"),
+      #                   c("style", "rounded,filled", "node"),
+      #                   c("color", "#000000", "node"),
+      #                   c("color", "#888888", "edge"),
+      #                   c("dir", "none", "edge"),
+      #                   c("headclip", "false", "edge"),
+      #                   c("tailclip", "false", "edge"),
+      #                   c("fillcolor", "#FFFFFF", "node"));
 
     if (!silent) {
       cat0("Successfully combined deductive code tree specifications into actual tree. Starting merging with inductive code trees.\n");
